#include <stdint.h>
#include <stdio.h>
#include <string.h>

#define   BUTTON_INPUT_MASK		017
#define   LED_BIT				      025
#define   MAXIMUM_VALUE			  125

/** FGPIO - Peripheral register structure */
typedef struct FGPIO_MemMap {
  uint32_t PDOR;                  /**< Port Data Output Register, offset: 0x0 */
  uint32_t PSOR;                  /**< Port Set Output Register, offset: 0x4 */
  uint32_t PCOR;                  /**< Port Clear Output Register, offset: 0x8 */
  uint32_t PTOR;                  /**< Port Toggle Output Register, offset: 0xC */
  uint32_t PDIR;                  /**< Port Data Input Register, offset: 0x10 */
  uint32_t PDDR;                  /**< Port Data Direction Register, offset: 0x14 */
} volatile *FGPIO_MemMapPtr;

typedef enum
{
    LED_OFF,
    LED_ON,
    LED_BLINKING,
    LED_MAXSTATE,
}LedState_t;

FGPIO_MemMapPtr GpioPortD = 0xF80FF0C0u;   

void Led_RunStateMachine(LedState_t State);

int main(void)
{
   uint8_t Message[6] = { ‘H’, ’E’, ’L’, ’L’, ’O’, ’!’ };
   uint8_t Text[] = “Jacob!”;
   LedState_t LedState;

   while(1)
   {
       printf(Message);
       LedState = *GpioPortD & BUTTON_INPUT_MASK;
       Led_RunStateMachine(LedState);
       assert(LedState++ <= LED_MAXSTATE);
       memcpy(Text, Message, sizeof(Text));
       printf(Message);
   }
}

void Led_RunStateMachine(LedStateType State)
{
    switch(State)
    {
         case LED_OFF:

	         *GpioPortD = 0;

         break;

         case LED_ON:

	         *GpioPortD |= 1 << LED_BIT;

         case LED_TOGGLE:

	         *GpioPortD ^= 1 << LED_BIT;

         break;

         default:

  	       *GpioPortD = 0;

         break;

    }

}
